from textwrap import wrap

import arrow
from clinicedc_constants import OTHER, YES
from django import template
from django.conf import settings
from django.utils.safestring import mark_safe
from edc_adverse_event.utils import get_adverse_event_app_label

register = template.Library()


format_ae_description_template_name = (
    f"{get_adverse_event_app_label()}/ae_initial_description.html"
)


@register.inclusion_tag(format_ae_description_template_name, takes_context=True)
def format_ae_description(context, ae_initial, wrap_length):
    formatted_sae_reason = "<BR>".join(wrap(ae_initial.sae_reason.name, wrap_length or 35))
    formatted_ae_description = "<BR>".join(wrap(ae_initial.ae_description, wrap_length or 35))
    context["utc_date"] = arrow.now().date()
    context["SHORT_DATE_FORMAT"] = settings.SHORT_DATE_FORMAT
    context["OTHER"] = OTHER
    context["YES"] = YES
    context["ae_initial"] = ae_initial
    context["sae_reason"] = mark_safe(formatted_sae_reason)  # nosec B308, B703  # noqa: S308
    context["ae_description"] = mark_safe(formatted_ae_description)  # nosec B308, B703  # noqa: S308
    return context



from clinicedc_constants import NO, OTHER
from edc_crf.crf_form_validator import CrfFormValidator


class HivExitReviewFormValidator(CrfFormValidator):
    def clean(self):
        self.required_if(NO, field="available", field_required="not_available_reason")
        self.required_if_not_none(
            "viral_load", "viral_load_date", field_required_evaluate_as_int=True
        )

        self.required_if_not_none("cd4", "cd4_date", field_required_evaluate_as_int=True)

        self.required_if(
            OTHER,
            field="current_arv_regimen",
            field_required="other_current_arv_regimen",
        )



# Generated by Django 5.1 on 2024-09-09 20:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0046_auto_20240829_0250"),
    ]

    operations = [
        migrations.CreateModel(
            name="ImpSubstitutions",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
                ("original_id", models.UUIDField(null=True)),
                ("sid", models.IntegerField(null=True, verbose_name="Original SID")),
                (
                    "dispensed_sid",
                    models.IntegerField(null=True, verbose_name="Dispensed SID"),
                ),
                (
                    "arm_match",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("not_evaluated", "Not evaluated"),
                        ],
                        default="not_evaluated",
                        max_length=15,
                    ),
                ),
                ("allocated_datetime", models.DateTimeField(null=True)),
                ("user_created", models.CharField(max_length=25)),
                ("user_modified", models.CharField(max_length=25)),
                ("modified", models.DateTimeField(null=True)),
            ],
            options={
                "verbose_name": "IMP Substitutions",
                "verbose_name_plural": "IMP Substitutions",
                "db_table": "imp_subjectitutions_view",
                "managed": False,
                "default_permissions": ("view", "export", "viewallsites"),
            },
        ),
    ]



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from .admin_site import meta_lists_admin
from .models import (
    ArvRegimens,
    BaselineSymptoms,
    Complications,
    DiabetesSymptoms,
    DiagnosticDevices,
    DmMedications,
    DmTreatments,
    HealthcareWorkers,
    Investigations,
    NonAdherenceReasons,
    OffstudyReasons,
    OiProphylaxis,
    SubjectVisitMissedReasons,
    Symptoms,
)


@admin.register(SubjectVisitMissedReasons, site=meta_lists_admin)
class SubjectVisitMissedReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Symptoms, site=meta_lists_admin)
class SymptomsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(ArvRegimens, site=meta_lists_admin)
class ArvRegimensAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(OffstudyReasons, site=meta_lists_admin)
class OffstudyReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(OiProphylaxis, site=meta_lists_admin)
class OiProphylaxisAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(BaselineSymptoms, site=meta_lists_admin)
class BaselineSymptomsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(DiabetesSymptoms, site=meta_lists_admin)
class DiabetesSymptomsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(NonAdherenceReasons, site=meta_lists_admin)
class NonAdherenceReasonsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(DmMedications, site=meta_lists_admin)
class DmMedicationsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(DmTreatments, site=meta_lists_admin)
class DmTreatmentsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Complications, site=meta_lists_admin)
class ComplicationsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Investigations, site=meta_lists_admin)
class InvestigationsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(HealthcareWorkers, site=meta_lists_admin)
class HealthcareWorkersAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(DiagnosticDevices, site=meta_lists_admin)
class DiagnosticDevicesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass



from django import forms
from edc_adverse_event.modelform_mixins import AeModelFormMixin
from edc_adverse_event.utils import validate_ae_initial_outcome_date
from edc_form_validators import FormValidator, FormValidatorMixin


class AeReviewFormValidator(FormValidator):
    def clean(self):
        pass


class AeReviewModelFormMixin(
    AeModelFormMixin,
    FormValidatorMixin,
):
    form_validator_cls = AeReviewFormValidator

    def clean(self):
        cleaned_data = super().clean()
        validate_ae_initial_outcome_date(self)
        return cleaned_data

    class Meta:
        help_text = {"subject_identifier": "(read-only)", "action_identifier": "(read-only)"}  # noqa: RUF012
        widgets = {  # noqa: RUF012
            "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
            "action_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
        }



# Generated by Django 5.0 on 2024-01-09 23:21

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0059_alter_icpreferral_managers_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalicpreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalicpreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartone",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningpartthree",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalscreeningparttwo",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectrefusal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectrefusal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="icpreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="icpreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjectrefusal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjectrefusal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
    ]



# from .icp_referral import IcpReferral
from .proxy_models import ScreeningPartOne, ScreeningPartThree, ScreeningPartTwo
from .signals import update_p3_ltfu_on_post_save
from .subject_refusal import SubjectRefusal
from .subject_screening import SubjectScreening

__all__ = [
    "ScreeningPartOne",
    "ScreeningPartThree",
    "ScreeningPartTwo",
    "SubjectRefusal",
    "SubjectScreening",
    "update_p3_ltfu_on_post_save",
]



# Generated by Django 5.1.5 on 2025-01-22 03:27

import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.lastname_field
import edc_consent.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_consent", "0030_auto_20250120_2114"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is &#x27;LASTNAME, FIRSTNAME&#x27;. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="is_incarcerated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If 'Yes' STOP participant cannot be consented.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_no],
                verbose_name="Is the participant under involuntary incarceration?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="witness_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text="Required only if participant is illiterate.<BR>Format is &#x27;LASTNAME, FIRSTNAME&#x27;. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Witness's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsentv1",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is &#x27;LASTNAME, FIRSTNAME&#x27;. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsentv1",
            name="is_incarcerated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If 'Yes' STOP participant cannot be consented.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_no],
                verbose_name="Is the participant under involuntary incarceration?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsentv1",
            name="witness_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text="Required only if participant is illiterate.<BR>Format is &#x27;LASTNAME, FIRSTNAME&#x27;. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Witness's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is &#x27;LASTNAME, FIRSTNAME&#x27;. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="is_incarcerated",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If 'Yes' STOP participant cannot be consented.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_no],
                verbose_name="Is the participant under involuntary incarceration?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="witness_name",
            field=django_crypto_fields.fields.lastname_field.LastnameField(
                blank=True,
                help_text="Required only if participant is illiterate.<BR>Format is &#x27;LASTNAME, FIRSTNAME&#x27;. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Witness's last and first name",
            ),
        ),
    ]



# see http://www.simonkrueger.com/2015/05/27/logging-django-apps-to-syslog.html
from pathlib import Path

import environ

__all__ = ["LOGGING", "LOGGING_FILE_LEVEL", "LOGGING_SYSLOG_LEVEL", "LOG_FOLDER"]

env = environ.Env()
env.read_env(".env")

LOG_FOLDER = Path(env.str("DJANGO_LOG_FOLDER"))
LOGGING_FILE_LEVEL = env.str("DJANGO_LOGGING_FILE_LEVEL")
LOGGING_SYSLOG_LEVEL = env.str("DJANGO_LOGGING_SYSLOG_LEVEL")


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_false": {"()": "django.utils.log.RequireDebugFalse"},
        "require_debug_true": {"()": "django.utils.log.RequireDebugTrue"},
        "ignore_specific_ip": {
            "()": "edc_utils.logging_filters.IgnoreSpecificIPDisallowedHost",
            "ip_to_ignore": "178.128.175.239",
        },
    },
    "formatters": {
        "verbose": {
            "format": (
                "[%(asctime)s] %(process)-5d %(thread)d "
                "%(name)-50s %(levelname)-8s %(message)s"
            ),
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
        "simple": {
            "format": "[%(asctime)s] %(name)s %(levelname)s %(message)s",
            "datefmt": "%d/%b/%Y %H:%M:%S",
        },
    },
    "handlers": {
        "file": {
            "level": LOGGING_FILE_LEVEL,
            "class": "logging.FileHandler",
            "filename": LOG_FOLDER / "edc.log",
            "formatter": "verbose",
        },
        "syslog": {
            "level": LOGGING_SYSLOG_LEVEL,
            "class": "logging.handlers.SysLogHandler",
            "facility": "local7",
            # "address": "/dev/log",
            "formatter": "verbose",
        },
    },
    "loggers": {
        "django": {
            "handlers": ["file"],
            "level": LOGGING_FILE_LEVEL,
            "propagate": True,
        },
        # root logger
        "": {"handlers": ["syslog"], "level": LOGGING_SYSLOG_LEVEL, "disabled": False},
        "meta-trial": {
            "handlers": ["syslog"],
            "level": LOGGING_SYSLOG_LEVEL,
            "propagate": False,
        },
    },
}



from clinicedc_constants import NOT_APPLICABLE
from django import forms
from edc_consent.modelform_mixins import ReviewFieldsModelFormMixin
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_utils.date import to_local

from ..consents import consent_v1_ext
from ..models import SubjectConsentV1Ext


class SubjectConsentV1ExtForm(
    SiteModelFormMixin, ReviewFieldsModelFormMixin, FormValidatorMixin, forms.ModelForm
):
    def clean(self):
        cleaned_data = super().clean()
        start_datetime = to_local(consent_v1_ext.start)
        if (
            cleaned_data.get("report_datetime")
            and cleaned_data.get("report_datetime") < start_datetime
        ):
            dt = start_datetime.date().strftime("%Y-%m-%d")
            raise forms.ValidationError(
                {"report_datetime": f"Cannot be before consent approval date {dt}."}
            )

        if cleaned_data.get("agrees_to_extension") == NOT_APPLICABLE:
            raise forms.ValidationError({"agrees_to_extension": "Invalid option"})
        return cleaned_data

    widgets = {  # noqa: RUF012
        "subject_identifier": forms.TextInput(attrs={"readonly": "readonly"}),
    }

    class Meta:
        model = SubjectConsentV1Ext
        fields = "__all__"
        labels = {  # noqa: RUF012
            "study_questions": (
                "I have answered all questions the participant had about the "
                "follow-up extension"
            ),
            "assessment_score": (
                "I have asked the participant questions about the follow-up extension and "
                "the participant has demonstrated understanding"
            ),
        }



from edc_crf.modeladmin_mixins import CrfStatusModelAdminMixin
from edc_model_admin.dashboard import (
    ModelAdminCrfDashboardMixin,
    ModelAdminSubjectDashboardMixin,
)
from edc_model_admin.mixins import ModelAdminProtectPiiMixin
from edc_sites.admin import SiteModelAdminMixin


class ModelAdminMixin(ModelAdminSubjectDashboardMixin):
    pass


class CrfModelAdminMixin(
    ModelAdminProtectPiiMixin,
    SiteModelAdminMixin,
    CrfStatusModelAdminMixin,
    ModelAdminCrfDashboardMixin,
):
    pass



# Generated by Django 5.0.8 on 2024-08-19 19:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0022_auto_20240819_1832"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="endpoints",
            index=models.Index(
                fields=["subject_identifier", "site"], name="meta_report_subject_a56b22_idx"
            ),
        ),
    ]



from django.contrib import admin
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from django_audit_fields import audit_fieldset_tuple
from edc_action_item.fieldsets import action_fieldset_tuple
from edc_action_item.modeladmin_mixins import ActionItemModelAdminMixin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import meta_prn_admin
from ..forms import OffScheduleForm
from ..models import EndOfStudy, OffSchedule


@admin.register(OffSchedule, site=meta_prn_admin)
class OffScheduleAdmin(
    SiteModelAdminMixin,
    ActionItemModelAdminMixin,
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = OffScheduleForm

    additional_instructions = mark_safe(  # noqa: S308
        render_to_string(
            "meta_prn/offschedule/additional_instructions.html",
            context=dict(verbose_name=EndOfStudy._meta.verbose_name),
        )
    )  # nosec B703, B308

    fieldsets = (
        (None, {"fields": ("subject_identifier", "offschedule_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    def get_list_display(self, request) -> tuple[str, ...]:
        list_display = super().get_list_display(request)
        custom_fields = ("subject_identifier", "dashboard", "offschedule_datetime")
        return custom_fields + tuple(f for f in list_display if f not in custom_fields)

    def get_list_filter(self, request) -> tuple[str, ...]:
        list_filter = super().get_list_filter(request)
        custom_fields = ("offschedule_datetime",)
        return custom_fields + tuple(f for f in list_filter if f not in custom_fields)



from decimal import Decimal

from clinicedc_constants import NO, PENDING, YES
from edc_form_validators import INVALID_ERROR

from meta_reports.models import GlucoseSummary


class EndpointValidatorMixin:
    def validate_endpoint_fields(self, performed: bool):
        if performed:
            is_endpoint = self.is_endpoint()
            if is_endpoint == YES and self.cleaned_data.get("endpoint_today") != YES:
                self.raise_validation_error(
                    {
                        "endpoint_today": (
                            "Participant has reached a study endpoint today. Expected YES"
                        )
                    },
                    INVALID_ERROR,
                )
            elif is_endpoint == PENDING and self.cleaned_data.get("endpoint_today") != PENDING:
                self.raise_validation_error(
                    {
                        "endpoint_today": (
                            "Participant has not reached a study endpoint today. "
                            "Expected to repeat FBG"
                        )
                    },
                    INVALID_ERROR,
                )

            elif is_endpoint == NO and self.cleaned_data.get("endpoint_today") != NO:
                self.raise_validation_error(
                    {"endpoint_today": "Participant has not reached a study endpoint today"},
                    INVALID_ERROR,
                )

    def is_endpoint(self):
        value = NO
        if (
            self.cleaned_data.get("fbg_value")
            and self.cleaned_data.get("ogtt_value")
            and self.cleaned_data.get("fbg_value") >= Decimal("7.0")
            and self.cleaned_data.get("ogtt_value") >= Decimal("11.1")
        ) or (
            self.cleaned_data.get("fbg_value")
            and self.cleaned_data.get("ogtt_value")
            and self.cleaned_data.get("fbg_value") < Decimal("7.0")
            and self.cleaned_data.get("ogtt_value") >= Decimal("11.1")
        ):
            value = YES
        elif (
            self.cleaned_data.get("fbg_value")
            and self.cleaned_data.get("ogtt_value")
            and self.cleaned_data.get("fbg_value") >= Decimal("7.0")
            and self.cleaned_data.get("ogtt_value") < Decimal("11.1")
        ):
            # is there a previous FBG>=7.0 in sequence or do you need to repeat?
            previous_obj = (
                GlucoseSummary.objects.filter(
                    subject_identifier=self.subject_identifier,
                    fbg_datetime__lt=self.cleaned_data.get("fbg_datetime"),
                )
                .order_by("fbg_datetime")
                .last()
            )
            if previous_obj and previous_obj.fbg_value >= Decimal("7.0"):
                value = YES
            else:
                # you need to schedule a repeat
                value = PENDING
        return value


class RepeatFbgDateValidatorMixin:
    def validate_repeat_fbg_date(self):
        if self.cleaned_data.get("repeat_fbg_date"):
            delta = (
                self.cleaned_data.get("repeat_fbg_date")
                - self.cleaned_data.get("fbg_datetime").date()
            )
            if delta.days < 7:
                self.raise_validation_error(
                    {"repeat_fbg_date": "Must be at least 7 days from the FBG date above"},
                    INVALID_ERROR,
                )
            if delta.days > 10:
                self.raise_validation_error(
                    {
                        "repeat_fbg_date": (
                            "Must be no more than 10 days from the FBG date above"
                        )
                    },
                    INVALID_ERROR,
                )



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    subquery = """
        select subject_identifier, v.visit_code, v.visit_code_sequence,cd4,
        cd4_date, crf.site_id, crf.user_created, crf.user_modified,
        crf.modified
        from meta_subject_patienthistory as crf
        left join meta_subject_subjectvisit as v on crf.subject_visit_id=v.id
        where cd4 is null or cd4_date is null
        """
    sql_view = SqlViewGenerator(
        report_model="meta_reports.patienthistorymissingbaselinecd4",
        ordering=["subject_identifier", "site_id"],
    )
    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 3.2.11 on 2022-03-09 19:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0098_auto_20220309_2106"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalurinepregnancy",
            name="notified",
            field=models.BooleanField(
                default=False,
                editable=False,
                help_text="Auto-updated by Pregnancy Notification PRN form",
            ),
        ),
        migrations.AddField(
            model_name="historicalurinepregnancy",
            name="notified_datetime",
            field=models.DateTimeField(
                editable=False,
                help_text="Auto-updated by Pregnancy Notification PRN form",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="urinepregnancy",
            name="notified",
            field=models.BooleanField(
                default=False,
                editable=False,
                help_text="Auto-updated by Pregnancy Notification PRN form",
            ),
        ),
        migrations.AddField(
            model_name="urinepregnancy",
            name="notified_datetime",
            field=models.DateTimeField(
                editable=False,
                help_text="Auto-updated by Pregnancy Notification PRN form",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalurinepregnancy",
            name="assay_date",
            field=models.DateField(blank=True, null=True, verbose_name="Urine βhCG date"),
        ),
        migrations.AlterField(
            model_name="urinepregnancy",
            name="assay_date",
            field=models.DateField(blank=True, null=True, verbose_name="Urine βhCG date"),
        ),
    ]



from clinicedc_constants import HIGH_PRIORITY
from edc_action_item.action import Action
from edc_action_item.site_action_items import site_action_items

from .constants import CONSENT_V1_EXTENSION_ACTION, RECONSENT_ACTION


class ReconsentAction(Action):
    name = RECONSENT_ACTION
    display_name = "Re-consent participant"
    reference_model = "meta_consent.subjectreconsent"
    priority = HIGH_PRIORITY
    show_on_dashboard = True
    show_link_to_changelist = True
    admin_site_name = "meta_consent_admin"
    create_by_user = False
    singleton = True
    instructions = (
        "Participant must be re-consented as soon as able. "
        "Participant's ICF was initially completed by next-of-kin."
    )

    def reopen_action_item_on_change(self):
        return False


class ConsentV1ExtensionAction(Action):
    name = CONSENT_V1_EXTENSION_ACTION
    display_name = "Ask to extend followup (required)"
    reference_model = "meta_consent.subjectconsentv1ext"
    priority = HIGH_PRIORITY
    show_on_dashboard = True
    show_link_to_changelist = True
    admin_site_name = "meta_consent_admin"
    show_link_to_add = True
    create_by_user = True
    singleton = True
    instructions = "Participant must complete as soon as able. "

    def reopen_action_item_on_change(self):
        return False


site_action_items.register(ReconsentAction)
site_action_items.register(ConsentV1ExtensionAction)



# Generated by Django 3.2.4 on 2021-07-16 05:13

import django.core.validators
import edc_model.validators.date
import edc_vitals.models.fields.weight
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0064_auto_20210715_2336"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalpatienthistory",
            name="dyslipidaemia_diagnosis",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Has the patient been diagnosed with dyslipidaemia?",
            ),
        ),
        migrations.AddField(
            model_name="historicalpatienthistory",
            name="on_dyslipidaemia_treatment",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Is the patient on treatment for dyslipidaemia?",
            ),
        ),
        migrations.AddField(
            model_name="patienthistory",
            name="dyslipidaemia_diagnosis",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                null=True,
                verbose_name="Has the patient been diagnosed with dyslipidaemia?",
            ),
        ),
        migrations.AddField(
            model_name="patienthistory",
            name="on_dyslipidaemia_treatment",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Is the patient on treatment for dyslipidaemia?",
            ),
        ),
        migrations.AlterField(
            model_name="followupvitals",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(null=True),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="hbsag",
            field=models.CharField(
                blank=True,
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("IND", "Indeterminate"),
                ],
                max_length=15,
                null=True,
                verbose_name="<u>HbSAg</u>",
            ),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="hbsag_date",
            field=models.DateField(
                blank=True,
                help_text="Approximate if not known",
                null=True,
                validators=[
                    edc_model.validators.date.date_is_past,
                    edc_model.validators.date.date_is_not_now,
                ],
                verbose_name="<i>HbSAg date</i>",
            ),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="hbsag_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Answer `YES` if `ever` performed and a result is available.",
                max_length=15,
                verbose_name="Was Hepatitis B Surface Antigen test performed?",
            ),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="hcv",
            field=models.CharField(
                blank=True,
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("IND", "Indeterminate"),
                ],
                max_length=15,
                null=True,
                verbose_name="<u>HCV</u>",
            ),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="hcv_date",
            field=models.DateField(
                blank=True,
                help_text="Approximate if not known",
                null=True,
                validators=[
                    edc_model.validators.date.date_is_past,
                    edc_model.validators.date.date_is_not_now,
                ],
                verbose_name="<i>HCV date</i>",
            ),
        ),
        migrations.AlterField(
            model_name="hepatitistest",
            name="hcv_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Answer `YES` if `ever` performed and a result is available.",
                max_length=15,
                verbose_name="Was the patient tested for Hepatitis C?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalfollowupvitals",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(null=True),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="hbsag",
            field=models.CharField(
                blank=True,
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("IND", "Indeterminate"),
                ],
                max_length=15,
                null=True,
                verbose_name="<u>HbSAg</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="hbsag_date",
            field=models.DateField(
                blank=True,
                help_text="Approximate if not known",
                null=True,
                validators=[
                    edc_model.validators.date.date_is_past,
                    edc_model.validators.date.date_is_not_now,
                ],
                verbose_name="<i>HbSAg date</i>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="hbsag_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Answer `YES` if `ever` performed and a result is available.",
                max_length=15,
                verbose_name="Was Hepatitis B Surface Antigen test performed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="hcv",
            field=models.CharField(
                blank=True,
                choices=[
                    ("POS", "Positive"),
                    ("NEG", "Negative"),
                    ("IND", "Indeterminate"),
                ],
                max_length=15,
                null=True,
                verbose_name="<u>HCV</u>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="hcv_date",
            field=models.DateField(
                blank=True,
                help_text="Approximate if not known",
                null=True,
                validators=[
                    edc_model.validators.date.date_is_past,
                    edc_model.validators.date.date_is_not_now,
                ],
                verbose_name="<i>HCV date</i>",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhepatitistest",
            name="hcv_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Answer `YES` if `ever` performed and a result is available.",
                max_length=15,
                verbose_name="Was the patient tested for Hepatitis C?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatienthistory",
            name="dyslipidaemia_rx",
            field=models.CharField(
                choices=[
                    ("atorvastatin", "Atorvastatin"),
                    ("rosuvastatin", "Rosuvastatin"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="What medication is the patient currently taking for dyslipidaemia?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="waist_circumference",
            field=models.DecimalField(
                decimal_places=1,
                help_text="in centimeters",
                max_digits=5,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(50.0),
                    django.core.validators.MaxValueValidator(175.0),
                ],
                verbose_name="Waist circumference",
            ),
        ),
        migrations.AlterField(
            model_name="historicalphysicalexam",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(null=True),
        ),
        migrations.AlterField(
            model_name="patienthistory",
            name="dyslipidaemia_rx",
            field=models.CharField(
                choices=[
                    ("atorvastatin", "Atorvastatin"),
                    ("rosuvastatin", "Rosuvastatin"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="What medication is the patient currently taking for dyslipidaemia?",
            ),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="waist_circumference",
            field=models.DecimalField(
                decimal_places=1,
                help_text="in centimeters",
                max_digits=5,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(50.0),
                    django.core.validators.MaxValueValidator(175.0),
                ],
                verbose_name="Waist circumference",
            ),
        ),
        migrations.AlterField(
            model_name="physicalexam",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(null=True),
        ),
    ]



# Generated by Django 5.0.1 on 2024-02-13 00:07

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "meta_subject",
            "0166_rename_dm_medications_other_dmreferralfollowup_other_dm_medications_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="dmreferralfollowup",
            old_name="healthcare_workers_other",
            new_name="other_healthcare_workers",
        ),
        migrations.RenameField(
            model_name="historicaldmreferralfollowup",
            old_name="healthcare_workers_other",
            new_name="other_healthcare_workers",
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_ae", "0009_auto_20220307_1929"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalaefollowup",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical AE Follow-up Report",
                "verbose_name_plural": "historical AE Follow-up Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalaeinitial",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical AE Initial Report",
                "verbose_name_plural": "historical AE Initial Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalaesusar",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical AE SUSAR Report",
                "verbose_name_plural": "historical AE SUSAR Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalaetmg",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical AE TMG Report",
                "verbose_name_plural": "historical AE TMG Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldeathreport",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Death Report",
                "verbose_name_plural": "historical Death Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldeathreporttmg",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Death Report TMG (1st)",
                "verbose_name_plural": "historical Death Report TMG (1st)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaldeathreporttmgsecond",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Death Report TMG (2nd)",
                "verbose_name_plural": "historical Death Report TMG (2nd)",
            },
        ),
        migrations.AlterField(
            model_name="historicalaefollowup",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalaeinitial",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalaesusar",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalaetmg",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldeathreport",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmg",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicaldeathreporttmgsecond",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]



from clinicedc_constants import (
    DEAD,
    DELIVERY,
    DIABETES,
    LTFU,
    NOT_APPLICABLE,
    NULL_STRING,
    OTHER,
    PREGNANCY,
    TOXICITY,
)
from django.db import models
from edc_action_item.models import ActionModelMixin
from edc_constants.choices import YES_NO_NA
from edc_model.models import BaseUuidModel
from edc_model.validators import date_not_future
from edc_offstudy.constants import (
    COMPLETED_FOLLOWUP,
    END_OF_STUDY_ACTION,
    LATE_EXCLUSION,
    WITHDRAWAL,
)
from edc_offstudy.model_mixins import OffstudyModelMixin
from edc_sites.model_mixins import SiteModelMixin
from edc_transfer.constants import TRANSFERRED

from meta_lists.models import OffstudyReasons

from ..choices import CLINICAL_WITHDRAWAL_REASONS, TOXICITY_WITHDRAWAL_REASONS
from ..constants import CLINICAL_WITHDRAWAL, COMPLETED_FOLLOWUP_48

# TODO: confirm all appointments are either new, incomplete or done
# TODO: take off study meds but coninue followup (WITHDRAWAL)
# TODO: follow on new schedule, if permanently off drug (Single 36m visit)

# TODO: add label for "End of followup as per protocol FEB2026"


class EndOfStudy(ActionModelMixin, SiteModelMixin, OffstudyModelMixin, BaseUuidModel):
    action_name = END_OF_STUDY_ACTION

    last_seen_date = models.DateField(
        verbose_name="Date patient was last seen",
        validators=[date_not_future],
        blank=False,
        null=True,
    )

    offstudy_reason = models.ForeignKey(
        OffstudyReasons,
        verbose_name="Reason patient was terminated from the study",
        on_delete=models.PROTECT,
        null=True,
        limit_choices_to={
            "name__in": [
                COMPLETED_FOLLOWUP,
                COMPLETED_FOLLOWUP_48,
                DIABETES,
                DELIVERY,
                PREGNANCY,
                CLINICAL_WITHDRAWAL,
                DEAD,
                LTFU,
                TOXICITY,
                TRANSFERRED,
                WITHDRAWAL,
                LATE_EXCLUSION,
                OTHER,
            ]
        },
    )

    other_offstudy_reason = models.TextField(
        verbose_name="If OTHER, please specify",
        max_length=500,
        blank=True,
        default=NULL_STRING,
    )

    # TODO: 6m off drug and duration ?? See SOP
    ltfu_date = models.DateField(
        verbose_name="Date lost to followup, if applicable",
        validators=[date_not_future],
        blank=True,
        null=True,
        help_text="A Loss to followup report must be on file",
    )

    death_date = models.DateField(
        verbose_name="Date of death, if applicable",
        validators=[date_not_future],
        blank=True,
        null=True,
        help_text="A Death report must be on file",
    )

    pregnancy_date = models.DateField(
        verbose_name="Date pregnancy known/UPT, if applicable",
        validators=[date_not_future],
        blank=True,
        null=True,
        help_text=(
            "A UPT CRF must be on file and participant not on the "
            "delivery schedule. Use UPT date or, if UPT not needed, "
            "use report date on last UPT CRF."
        ),
    )

    delivery_date = models.DateField(
        verbose_name="Date of delivery, if applicable",
        validators=[date_not_future],
        blank=True,
        null=True,
        help_text=(
            "A Delivery CRF must be on file. Use delivery date, "
            "if reported, or report date from Delivery CRF"
        ),
    )

    clinical_withdrawal_reason = models.CharField(
        verbose_name=(
            "If the patient was withdrawn on CLINICAL grounds, please specify PRIMARY reason"
        ),
        max_length=25,
        choices=CLINICAL_WITHDRAWAL_REASONS,
        default=NOT_APPLICABLE,
    )

    clinical_withdrawal_reason_other = models.TextField(
        verbose_name="If withdrawn for 'other' condition, please explain",
        max_length=500,
        blank=True,
        default=NULL_STRING,
    )

    clinical_withdrawal_investigator_decision = models.TextField(
        verbose_name="If withdrawl was an 'investigator decision', please explain ...",
        max_length=500,
        blank=True,
        default=NULL_STRING,
    )

    toxicity_withdrawal_reason = models.CharField(
        verbose_name=" If the patient experienced an unacceptable toxicity', please explain",
        max_length=25,
        choices=TOXICITY_WITHDRAWAL_REASONS,
        default=NOT_APPLICABLE,
    )

    toxicity_withdrawal_reason_other = models.TextField(
        verbose_name="If 'other toxicity', please specify ...",
        max_length=500,
        blank=True,
        default=NULL_STRING,
    )

    transfer_date = models.DateField(
        verbose_name="Date of transfer, if applicable",
        validators=[date_not_future],
        blank=True,
        null=True,
        help_text="A Transfer form must be on file.",
    )

    transferred_consent = models.CharField(
        verbose_name="If transferred, has the patient provided consent to be followed-up?",
        choices=YES_NO_NA,
        max_length=15,
        default=NOT_APPLICABLE,
    )

    comment = models.TextField(
        verbose_name="Please provide further details if possible",
        max_length=500,
        blank=True,
        default=NULL_STRING,
    )

    class Meta(OffstudyModelMixin.Meta):
        verbose_name = "End of Study"
        verbose_name_plural = "End of Study"



from clinicedc_constants import NULL_STRING, YES
from django.db import models
from edc_constants.choices import YES_NO
from edc_crf.model_mixins import SingletonCrfModelMixin
from edc_model.models import BaseUuidModel

from meta_lists.models import ArvRegimens

from ..model_mixins import ArvReviewModelMixin, CrfModelMixin


class HivExitReview(SingletonCrfModelMixin, ArvReviewModelMixin, CrfModelMixin, BaseUuidModel):
    available = models.CharField(
        verbose_name="Are HIV test result and treatment information available?",
        max_length=25,
        default=YES,
        choices=YES_NO,
    )

    not_available_reason = models.TextField(
        verbose_name="If not available, please explain ...",
        default=NULL_STRING,
        blank=True,
    )

    current_arv_regimen = models.ForeignKey(
        ArvRegimens,
        on_delete=models.PROTECT,
        related_name="last_arv_regimen",
        verbose_name="Which antiretroviral therapy regimen is the patient currently on?",
        null=True,
        blank=True,
    )

    comment = models.TextField(
        verbose_name="Any other comment?",
        default=NULL_STRING,
        blank=True,
        help_text="May be left blank.",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "HIV Exit Review"
        verbose_name_plural = "HIV Exit Review"



# Generated by Django 5.1 on 2024-09-11 00:27

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0048_auto_20240909_2338"),
    ]

    operations = [
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.imp_subjectitutions_view' as `report_model` from (SELECT s.subject_identifier, s.sid, s.dispensed_sid, arm_match, s.report_datetime, r.allocated_datetime, s.site_id, s.user_created, s.user_modified, s.modified, s.id AS original_id FROM meta_pharmacy_substitutions AS s LEFT JOIN meta_rando_randomizationlist AS r ON r.subject_identifier = s.subject_identifier ORDER BY s.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "imp_subjectitutions_view",
                engine="django.db.backends.mysql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, now() as `created`, 'meta_reports.imp_subjectitutions_view' as `report_model` from (SELECT s.subject_identifier, s.sid, s.dispensed_sid, arm_match, r.allocated_datetime, s.site_id, s.user_created, s.user_modified, s.modified, s.id AS original_id FROM meta_pharmacy_substitutions AS s LEFT JOIN meta_rando_randomizationlist AS r ON r.subject_identifier = s.subject_identifier ORDER BY s.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "imp_subjectitutions_view",
                engine="django.db.backends.mysql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'meta_reports.imp_subjectitutions_view' as report_model from (SELECT s.subject_identifier, s.sid, s.dispensed_sid, arm_match, s.report_datetime, r.allocated_datetime, s.site_id, s.user_created, s.user_modified, s.modified, s.id AS original_id FROM meta_pharmacy_substitutions AS s LEFT JOIN meta_rando_randomizationlist AS r ON r.subject_identifier = s.subject_identifier ORDER BY s.subject_identifier NULLS FIRST) as A ORDER BY subject_identifier, site_id",
                "imp_subjectitutions_view",
                engine="django.db.backends.postgresql",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, get_random_uuid() as id, now() as created, 'meta_reports.imp_subjectitutions_view' as report_model from (SELECT s.subject_identifier, s.sid, s.dispensed_sid, arm_match, r.allocated_datetime, s.site_id, s.user_created, s.user_modified, s.modified, s.id AS original_id FROM meta_pharmacy_substitutions AS s LEFT JOIN meta_rando_randomizationlist AS r ON r.subject_identifier = s.subject_identifier ORDER BY s.subject_identifier NULLS FIRST) as A ORDER BY subject_identifier, site_id",
                "imp_subjectitutions_view",
                engine="django.db.backends.postgresql",
            ),
            atomic=False,
        ),
        django_db_views.operations.ViewRunPython(
            code=django_db_views.migration_functions.ForwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'meta_reports.imp_subjectitutions_view' as report_model from (SELECT s.subject_identifier, s.sid, s.dispensed_sid, arm_match, s.report_datetime, r.allocated_datetime, s.site_id, s.user_created, s.user_modified, s.modified, s.id AS original_id FROM meta_pharmacy_substitutions AS s LEFT JOIN meta_rando_randomizationlist AS r ON r.subject_identifier = s.subject_identifier ORDER BY s.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "imp_subjectitutions_view",
                engine="django.db.backends.sqlite3",
            ),
            reverse_code=django_db_views.migration_functions.BackwardViewMigration(
                "select *, uuid() as id, datetime() as created, 'meta_reports.imp_subjectitutions_view' as report_model from (SELECT s.subject_identifier, s.sid, s.dispensed_sid, arm_match, r.allocated_datetime, s.site_id, s.user_created, s.user_modified, s.modified, s.id AS original_id FROM meta_pharmacy_substitutions AS s LEFT JOIN meta_rando_randomizationlist AS r ON r.subject_identifier = s.subject_identifier ORDER BY s.subject_identifier) as A ORDER BY subject_identifier, site_id",
                "imp_subjectitutions_view",
                engine="django.db.backends.sqlite3",
            ),
            atomic=False,
        ),
    ]
